# -*- coding: utf-8 -*-

import time
import datetime

import dateutil
import dateutil.parser

from pymongo import UpdateOne

from em_tasks import logger, get_mongo
from em_tasks.spapi.spapi_catalog_items_parser import SpapiCatalogItemsParser



class SpapiUpdateCatalogItemsTask():
  def __init__(self, spapi, product_service, marketplace, asins, error_service=None):
    self.spapi = spapi
    self.product_service = product_service
    self.error_service = error_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.mongo_client = get_mongo()

  def save_asins_to_mongodb(self, asins):
    if asins:
        operations = [UpdateOne({"_id": asin, "asin": asin}, {"$setOnInsert": {"asin": "asin"}}, upsert = True) for asin in asins]
        self.mongo_client.client[f"{self.marketplace}_failed_asins"].bulk_write(operations)

  def run(self):
    cur_time = datetime.datetime.strftime(datetime.datetime.utcnow(), '%Y-%m-%dT%H:%M:%S')
    indice_name = 'amz_products_api_{}'.format(self.marketplace)

    products_info = self.search_catalog_items(self.spapi, self.marketplace, self.asins)
    if not products_info:
      if self.error_service:
        self.error_service.save_no_product_info_asins(
          "product_info", self.asins, self.marketplace
        )
      return

    for _, product_info in products_info.items():
      product_info['_id'] = product_info['asin']
      product_info['time'] = cur_time
    try:
      self.product_service.save_products(indice_name, list(products_info.values()))
    except Exception as e:
      logger.warning('[ProductSaveToServiceError] %s', products_info)
      logger.exception(e)
      raise e

  def search_catalog_items(self, spapi, marketplace, asins):
    response = spapi.search_catalog_items(asins, marketplace=marketplace)
    if not response:
      return

    return SpapiCatalogItemsParser.parse(response)
